from .mask import apply_json_mask, should_include_variable
from .parsing import parse_fields


__project__ = 'jsonmask_ng'
__version__ = '0.4.0'

VERSION = "{0} v{1}".format(__project__, __version__)
